
function _list_count(self)
	return getn(self.items, item)
end

function _list_indexof(self, name)
	for i = 1, getn(self.items) do
		if strlower(name) == strlower(self.items[i].name) then
			return i
		end
	end
	return false
end

function _list_add(self, item)
	tinsert(self.items, item)
end



-- *** modulelist object ***

-- methods

-- constructor
function new_modulelist()
	local new = { }
	new.count = _list_count
	new.indexof = _list_indexof
	new.add = _list_add
	new.items = { }
	new.what = "modulelist"
	return new
end



-- *** optable object ***

-- methods
function _optable_load(self, filename)
	self.items = { }
	local file = openfile(filename, "r")
	if file then
		local line = read(file)
		while line do
			local s, e, name = strfind(line, "(%S+)%|%d+%|.*")
			local s, e, levelstr = strfind(line, "%S+%|(%d+)%|.*")
			local s, e, privilegestr = strfind(line, "%S+%|%d+%|(.*)")
			if name and levelstr and privilegestr then
				local level = tonumber(levelstr)
				if level < 1 then
					level = 1
				end
				if owner and (level >= ownerlevel) then
					level = ownerlevel - 1
				elseif level > ownerlevel then
					level = ownerlevel
				end
				local privileges = parselist(privilegestr)
				local item = {
					name = name,
					level = level,
					privileges = privileges
				}
				self:add(item)
				if level == ownerlevel then
					owner = self:count()
				end
			end
			line = read(file)
		end
		closefile(file)
	end
end

function _optable_save(self, filename)
	local file = openfile(filename, "w")
	for i = 1, getn(self.items) do
		local privilegestr
		if self.items[i].privileges == "*" then
			privilegestr = "*"
		else
			privilegestr = makelist(self.items[i].privileges)
		end
		write(file, self.items[i].name.."|"..self.items[i].level.."|"..privilegestr.."\n")
	end
	closefile(file)
end

function _optable_setlevel(self, name, level)
	index = self:indexof(name)
	if index then
		self.items[index].level = level
	else
		local item = {
			name = name,
			level = level,
			privileges = { }
		}
		self:add(item)
	end
end

function _optable_setprivileges(self, name, privileges)
	index = self:indexof(name)
	if index then
		self.items[index].privileges = privileges
	else
		local item = {
			name = name,
			level = 0,
			privileges = privileges
		}
		self:add(item)
	end
end

function _optable_getlevel(self, name)
	local index = self:indexof(name)
	if index then
		return self.items[index].level
	else
		return 0
	end
end

function _optable_getprivileges(self, name)
	local index = self:indexof(name)
	if index then
		return self.items[index].privileges
	else
		return { }
	end
end

-- constructor
function new_optable()
	local new = { }
	new.count = _list_count
	new.indexof = _list_indexof
	new.add = _list_add
	new.load = _optable_load
	new.save = _optable_save
	new.setlevel = _optable_setlevel
	new.setprivileges = _optable_setprivileges
	new.getlevel = _optable_getlevel
	new.getprivileges = _optable_getprivileges
	new.items = { }
	new.what = "optable"
	return new
end



-- *** bot object ***

-- methods
function _bot_pm(self, user, text)
	if user then
		user:SendPM(self.botname, text)
	end
end

function _bot_chat(self, text)
	SendToAll(self.botname, text)
end

function _bot_delayedchat(self, text)
	item = {
		name = self.botname,
		text = text
	}
	tinsert(chatqueue, item)
end

function _bot_chatfor(self, user, text)
	user:SendData("<"..self.botname.."> "..text)
end

function _bot_kick(self, user, msg)
	self:chat("Kickuju "..user.sName.."! ("..msg..")")
	self:pm(user, "Kicked! ("..msg..")")
	user:Disconnect()
end

function _bot_enter(self)
	frmHub:RegBot(self.botname)
end

function _bot_leave(self)
	frmHub:UnregBot(self.botname)
end

-- constructor
function new_bot(botname)
	local new = { }
	new.botname = botname
	new.pm = _bot_pm
	new.chat = _bot_chat
	new.delayedchat = _bot_delayedchat
	new.chatfor = _bot_chatfor
	new.kick = _bot_kick
	new.sendwelcome = _bot_sendwelcome
	new.enter = _bot_enter
	new.leave = _bot_leave
	return new
end