commands = {
	-- debugging
	hi = function(user, arg)
		return "Hello "..user.sName
	end,

	-- public
	help = function(user, arg)
		local s, e, topic = strfind(arg, "(%S+)")
		if topic then
			topic = strlower(topic)
			if help[topic] then
				return help[topic]
			else
				local vars = {
					topic = topic
				}
				return replacevars(messages.topicnotfound, vars)
			end
		else
			return help.default
		end
	end,
	operators = function(user, arg)
		local list = ""
		for i = 1, ops:count() do
			list = list.."\013\010"..ops.items[i].name
			if user.bOperator then
				list = list.." ("..ops.items[i].level.."; "..makelist(ops.items[i].privileges)..")"
			end
		end
		local vars = {
			list = list
		}
		return replacevars(messages.operators, vars)
	end,
	time = function(user, arg)
		local vars = {
			time = date("%H:%M")
		}
		return replacevars(messages.time, vars)
	end,
	whoami = function(user, arg)
		return whoisinfo(user)
	end,
	owner = function(user, arg)
		if owner then
			vars = {
				name = ops.items[owner].name
			}
			return replacevars(messages.owner, vars)
		end
	end,
	about = function(user, arg)
		local list = ""
		for index = 1, getn(modules.items) do
			if modules.items[index].getinfo then
				info = modules.items[index].getinfo()
				list = list.."\013\010 - "..modules.items[index].name.." ("..info.name..")"
			else
				list = list.."\013\010 - "..modules.items[index].name
			end
		end
		local vars = {
			list = list
		}
		return "\013\010"..aboutinfo.." by wintermute\013\010"..replacevars(messages.modules, vars)
	end,
	
	-- operators only
	say = function(user, arg)
		if user.bOperator then
			local s, e, text = strfind(arg, "(.+)")
			if text then
				mainbot:chat(text)
			else
				return messages.error_invalidparams
			end
		end
	end,
	whois = function(user, arg)
		if user.bOperator then
			local s, e, name = strfind(arg, "(%S+)")
			if name then
				local targuser = GetItemByName(name)
				if targuser then
					return whoisinfo(targuser)
				else
					return messages.error_usernotfound
				end
			else
				return messages.error_invalidparams
			end
		end
	end,
	redirect = function(user, arg)
		if user.bOperator then
			local s, e, address = strfind(arg, "(%S+)")
			if address then
				if bot.ops.items[curop].privileges.hubadmin then
					frmHub:SetRedirectAddress(address)
					return "Adresa pro redirect nastavena na "..address.."!"
				else
					return messages.error_noprivilege
				end
			else
				return "Adresa pro redirect je "..frmHub:GetRedirectAddress()
			end
		end
	end,
	maxusers = function(user, arg)
		if user.bOperator then
			local s, e, maxusers = strfind(arg, "(%d+)")
			if maxusers then
				if ops.items[curop].privileges.hubadmin then
					frmHub:SetMaxUsers(maxusers)
					return "Maximalni pocet uzivatelu nastaven na "..maxusers.."!"
				else
					return messages.error_noprivilege
				end
			else
				return "Maximalni pocet uzivatelu je "..frmHub:GetMaxUsers()
			end
		end
	end,
	level = function(user, arg)
		if user.bOperator then
			s, e, name = strfind(arg, "(%S+)")
			s, e, levelstr = strfind(arg, "%S+%s+(%d+)")
			if name then
				if levelstr then
					if hasprivilege(user, "opadmin") and levelof(user) > 0 then
						if ops:getlevel(name) < levelof(user) then
							local level = tonumber(levelstr)
							if level < 0 then
				 				level = 0
							end
							if level >= levelof(user) then
				 				level = levelof(user) - 1
							end
							ops:setlevel(name, level)
							saveops()
							local vars = {
								name = name,
								level = level
							}
							return replacevars(messages.newlevel, vars)
						else
							return messages.error_leveltoolow
						end
					else
						return messages.error_noprivilege
					end
				else
					local vars = {
						name = name,
						level = ops:getlevel(name)
					}
					return replacevars(messages.level, vars)
				end
			else
				return messages.error_invalidparams
			end
		end
	end,
	privileges = function(user, arg)
		if user.bOperator then
			s, e, name = strfind(arg, "(%S+)")
			s, e, privilegestr = strfind(arg, "%S+%s+(.*)")
			if name then
				local privileges
				if privilegestr then
		 			privileges = parselist(privilegestr)
				end
				if privileges then
					if hasprivilege(user, "opadmin") then
						if ops:getlevel(name) < levelof(user) then
							local newprivileges = { }
	 						for privilege, value in possprivileges do
		 						if hasprivilege(user, privilege) then
									newprivileges[privilege] = privileges[privilege]
								end
	 						end
	 						ops:setprivileges(name, newprivileges)
							ops:save(path.."ops.dat")
							local vars = {
								name = name,
								privileges = makelist(newprivileges)
							}
							return replacevars(messages.newprivileges, vars)
						else
							return messages.error_leveltoolow
						end
					else
						return messages.error_noprivilege
					end
				else	
					local vars = {
						name = name,
						privileges = makelist(ops:getprivileges(name))
					}
					return replacevars(messages.privileges, vars)
				end
			else
				return messages.error_invalidparams
			end
		end
	end
}